//
//  WMFlowLayout.m
//  自定义布局
//
//  Created by wangmiao on 16/4/29.
//  Copyright © 2016年 wangmiao. All rights reserved.
//

#import "WMFlowLayout.h"

@implementation WMFlowLayout

- (instancetype)init
{
    if (self = [super init]) {
        
    }
    return self;
}

// 布局的一些初始化可以在这个方法里面写,调用频率很高,不要做太过于复杂度的操作
- (void)prepareLayout
{
    [super prepareLayout];
//    NSLog(@"%s", __func__);
    CGFloat insert = (self.collectionView.frame.size.width - self.itemSize.width) * 0.5;
    self.sectionInset = UIEdgeInsetsMake(0, insert, 0, insert);
}

// 在collectionView滚动的实时更新
- (BOOL)shouldInvalidateLayoutForBoundsChange:(CGRect)newBounds
{
    return YES;
}

- (NSArray<UICollectionViewLayoutAttributes *> *)layoutAttributesForElementsInRect:(CGRect)rect
{
    // 计算中心轴的X
    CGFloat centerX = self.collectionView.contentOffset.x + self.collectionView.frame.size.width * 0.5;
    
    
    NSArray *array = [super layoutAttributesForElementsInRect:rect];
    for (UICollectionViewLayoutAttributes *atts in array) {
        /**
         *计算与中心轴之间的距离，通过距离修改scale
         */
        CGFloat distance = ABS(atts.center.x - centerX);
        CGFloat scale = 1 - distance / self.collectionView.frame.size.width;
        atts.transform =CGAffineTransformMakeScale(scale, scale);
    }
    
    return array;
}

- (CGPoint)targetContentOffsetForProposedContentOffset:(CGPoint)proposedContentOffset withScrollingVelocity:(CGPoint)velocity
{
//    NSLog(@"%s", __func__);
    CGRect rect;
    rect.origin.x = proposedContentOffset.x;
    rect.origin.y = 0;
    rect.size = self.collectionView.frame.size;
    
    // 计算轴心距
    CGFloat centerX = proposedContentOffset.x + self.collectionView.frame.size.width * 0.5;
    
    // 存放最小的轴心距距离
    CGFloat minD = MAXFLOAT;
    NSArray *array = [super layoutAttributesForElementsInRect:rect];
    for (UICollectionViewLayoutAttributes *atts in array) {
        if (ABS(minD) > ABS(atts.center.x - centerX)) {
            minD = atts.center.x - centerX;
        }
    }
    
    proposedContentOffset.x += minD;
    return proposedContentOffset;
}

@end
